@@ -31,12 +31,7 @@
 #define Py_TYPE(o) ((o)->ob_type)
 #endif
 
-#if PY_MAJOR_VERSION >= 3
-#define PY3K
 #define PyNative_FromString PyUnicode_FromString
-#else
-#define PyNative_FromString PyString_FromString
-#endif
 
 static PyObject *str__dict__, *str__implemented__, *strextends;
 static PyObject *BuiltinImplementationSpecifications, *str__provides__;
@@ -350,6 +345,7 @@ Spec_traverse(Spec* self, visitproc visit, void* arg)
 {
     Py_VISIT(self->_implied);
     Py_VISIT(self->_dependents);
+    Py_VISIT(self->_bases);
     Py_VISIT(self->_v_attrs);
     Py_VISIT(self->__iro__);
     Py_VISIT(self->__sro__);
@@ -361,6 +357,7 @@ Spec_clear(Spec* self)
 {
     Py_CLEAR(self->_implied);
     Py_CLEAR(self->_dependents);
+    Py_CLEAR(self->_bases);
     Py_CLEAR(self->_v_attrs);
     Py_CLEAR(self->__iro__);
     Py_CLEAR(self->__sro__);
@@ -524,8 +521,10 @@ OSD_descr_get(PyObject *self, PyObject *inst, PyObject *cls)
     return getObjectSpecification(NULL, cls);
 
   provides = PyObject_GetAttr(inst, str__provides__);
-  if (provides != NULL)
+  /* Return __provides__ if we got it, or return NULL and propagate non-AttributeError. */
+  if (provides != NULL || !PyErr_ExceptionMatches(PyExc_AttributeError))
     return provides;
+
   PyErr_Clear();
   return implementedBy(NULL, cls);
 }
@@ -678,7 +677,7 @@ static PyTypeObject CPBType = {
 
 /*
     def __adapt__(self, obj):
-        """Adapt an object to the reciever
+        """Adapt an object to the receiver
         """
         if self.providedBy(obj):
             return obj
@@ -759,10 +758,6 @@ __adapt__(PyObject *self, PyObject *obj)
   return Py_None;
 }
 
-#ifndef PY3K
-typedef long Py_hash_t;
-#endif
-
 typedef struct {
     Spec spec;
     PyObject* __name__;
@@ -772,7 +767,7 @@ typedef struct {
 
 static struct PyMethodDef ib_methods[] = {
   {"__adapt__", (PyCFunction)__adapt__, METH_O,
-   "Adapt an object to the reciever"},
+   "Adapt an object to the receiver"},
   {NULL,                NULL}           /* sentinel */
 };
 
@@ -1251,11 +1246,7 @@ _lookup(lookup *self,
 {
   PyObject *result, *key, *cache;
   result = key = cache = NULL;
-#ifdef PY3K
   if ( name && !PyUnicode_Check(name) )
-#else
-  if ( name && !PyString_Check(name) && !PyUnicode_Check(name) )
-#endif
   {
     PyErr_SetString(PyExc_ValueError,
                     "name is not a string or unicode");
@@ -1263,7 +1254,7 @@ _lookup(lookup *self,
   }
 
   /* If `required` is a lazy sequence, it could have arbitrary side-effects,
-     such as clearing our caches. So we must not retreive the cache until
+     such as clearing our caches. So we must not retrieve the cache until
      after resolving it. */
   required = PySequence_Tuple(required);
   if (required == NULL)
@@ -1347,11 +1338,7 @@ _lookup1(lookup *self,
 {
   PyObject *result, *cache;
 
-#ifdef PY3K
   if ( name && !PyUnicode_Check(name) )
-#else
-  if ( name && !PyString_Check(name) && !PyUnicode_Check(name) )
-#endif
   {
     PyErr_SetString(PyExc_ValueError,
                     "name is not a string or unicode");
@@ -1423,11 +1410,7 @@ _adapter_hook(lookup *self,
 {
   PyObject *required, *factory, *result;
 
-#ifdef PY3K
   if ( name && !PyUnicode_Check(name) )
-#else
-  if ( name && !PyString_Check(name) && !PyUnicode_Check(name) )
-#endif
   {
     PyErr_SetString(PyExc_ValueError,
                     "name is not a string or unicode");
@@ -1994,7 +1977,6 @@ static struct PyMethodDef m_methods[] = {
   {NULL,         (PyCFunction)NULL, 0, NULL}            /* sentinel */
 };
 
-#if  PY_MAJOR_VERSION >= 3
 static char module_doc[] = "C optimizations for zope.interface\n\n";
 
 static struct PyModuleDef _zic_module = {
@@ -2008,20 +1990,14 @@ static struct PyModuleDef _zic_module = {
         NULL,
         NULL
 };
-#endif
 
 static PyObject *
 init(void)
 {
   PyObject *m;
 
-#if  PY_MAJOR_VERSION < 3
-#define DEFINE_STRING(S) \
-  if(! (str ## S = PyString_FromString(# S))) return NULL
-#else
 #define DEFINE_STRING(S) \
   if(! (str ## S = PyUnicode_FromString(# S))) return NULL
-#endif
 
   DEFINE_STRING(__dict__);
   DEFINE_STRING(__implemented__);
@@ -2071,13 +2047,7 @@ init(void)
   if (PyType_Ready(&VerifyingBase) < 0)
     return NULL;
 
-  #if PY_MAJOR_VERSION < 3
-  /* Create the module and add the functions */
-  m = Py_InitModule3("_zope_interface_coptimizations", m_methods,
-                     "C optimizations for zope.interface\n\n");
-  #else
   m = PyModule_Create(&_zic_module);
-  #endif
   if (m == NULL)
     return NULL;
 
@@ -2101,17 +2071,10 @@ init(void)
 }
 
 PyMODINIT_FUNC
-#if PY_MAJOR_VERSION < 3
-init_zope_interface_coptimizations(void)
-{
-  init();
-}
-#else
 PyInit__zope_interface_coptimizations(void)
 {
   return init();
 }
-#endif
 
 #ifdef __clang__
 #pragma clang diagnostic pop
